function [outputCipher,authTag] = GCM_AEfltrad(secretKey,iv,plainTextInput,aad,sn256,car256)
%This is a modified standard AES-GCM implementation, modified by applying a patented 256-state FLT
% transformation and a patent pending radix-256 transformation
% While the basic AES-GCM implementation is IP-free, this particular
% implementation is not!
% You are only allowed to apply this program for
% testing/trial/education/research purposes. 
% Use for operational encryption is strictly prohibited.
% Certain aspects of the present implementation may be covered by patents:
% US 12,143,468; US 11,093,213; 12,056,549;US 11,336,425 and pending
% patent applications.

% Algorithm for the Authentiated Encryption Function
% The patent rights are owned by LCIP JV and include: US 
% Galois Counter Mode (GCM) Block Cipher uses AES-128,192,256 based in key
% length. Implimentation based on NIST Special Publication 800-38D.
% Inputs: secretKey - hexidecimal (128,192, or 256 bits), iv -
% initialization vector (random 96-bit hexidecimal), plainTextInput -
% plaintext message of input that needs to be encrypted, aad - additional
% authenticated data (plaintext string).  Outputs: outputCipher - 
% hexidecimal encrypted Cipher, authTag - authentication tag (hexidecimal).
% IT IS ESSENTIAL THAT THE INITIALIZATION VECTOR BE DIFFERENT FOR EACH
% ENCRYPTION USING THE SAME SECRET KEY. INITIALIZATION VECTOR GENERATION
% CAN BE INCORPORATED INTO THE CODE AND SHOULD FOLLOW NIST GUIDELINES.
%
% David Hill
% Version 1.0.0
% Date: 1 December 2021
%

H=Cipherfltrad(secretKey,zeros(1,128),sn256,car256);%Hash subkey
t=96;%authentication tag bit-length (fixed for this code)
iv=dec2bin(hex2dec(reshape(iv,2,[])'),8)';
iv=iv(:)'-'0';%binary array
p=dec2bin(double(plainTextInput)',8)';
p=p(:)'-'0';%binary array
a=dec2bin(double(aad)',8)';
a=a(:)'-'0';%binary array
J=[iv,zeros(1,31),1];
C=GCTRfltrad(secretKey,inc(32,J),p,sn256,car256);
u=128*ceil(length(C)/128)-length(C);
v=128*ceil(length(a)/128)-length(a);
S=ghash([a,zeros(1,v),C,zeros(1,u),dec2bin(length(a),64)-'0',dec2bin(length(C),64)-'0'],H);
T=GCTRfltrad(secretKey,J,S,sn256,car256);
T=T(1:t);
T=num2str(T);
T=T(T~=' ');
T=dec2hex(bin2dec(reshape(T,8,[])'))';
authTag=lower(T(:))';%convert T to hex
C=num2str(C);
C=C(C~=' ');
C=dec2hex(bin2dec(reshape(C,8,[])'))';
outputCipher=lower(C(:))';%convert C to hex